@@ -90,6 +90,14 @@ siena08 <- function(..., projname="sienaMeta", bound=5)
         x1 <- x[!is.na(x$theta) & !is.na(x$se) & x$se < bound,]
         if (any(x1$theta != 0))
         {
+            if (sum((x1$se < bound)) >= 3)
+            {
+                check.correl <- cor.test(x1$theta, x1$se)
+            }
+            else
+            {
+                check.correl <- data.frame(estimate=NA, p.value=NA)
+            }
             regfit <- iwlsm(theta ~ 1, psi=psi.iwlsm, data=x1,
                             ses=x1$se^2)
             regfit$terms <- NA
@@ -105,7 +113,9 @@ siena08 <- function(..., projname="sienaMeta", bound=5)
             cjminus <- -2 * sum(pnorm(x1$theta / x1$se, log=TRUE))
             cjplusp <- 1 - pchisq(cjplus, 2 * nrow(x1))
             cjminusp <- 1 - pchisq(cjminus, 2 * nrow(x1))
-            ret1 <- list(regfit=regfit, regsummary=regsummary,
+            ret1 <- list(cor.est=check.correl$estimate,
+                         cor.pval=check.correl$p.value,
+                         regfit=regfit, regsummary=regsummary,
                          Tsq=Tsq, pTsq=1 - pchisq(Tsq, nrow(x1) - 1),
                          tratio=tratio,
                          ptratio=2 * pnorm(abs(tratio), lower.tail=FALSE),
@@ -145,7 +155,7 @@ siena08 <- function(..., projname="sienaMeta", bound=5)
     meta$thetadf <- mydf
     class(meta) <- "sienaMeta"
     meta$projname <- projname
-    meta$bound <- 5
+    meta$bound <- bound
     ## count the score tests
     meta$scores <- by(mydf, mydf$effects, function(x)
                       any(!is.na(x$scoretests)))
@@ -181,7 +191,7 @@ print.sienaMeta <- function(x, file=FALSE, ...)
     nProjects <- length(projnames)
     effects <- unique(x$thetadf$effects)
     nEffects <- length(effects)
-## results
+    ## results
 
     ## estimates
 
@@ -208,16 +218,39 @@ print.sienaMeta <- function(x, file=FALSE, ...)
        Report(c(" ", y$n1, " datasets used.\n\n"), sep="", outf)
        if (y$n1 > 0)
        {
-           Report(c("\nTest that all parameters are 0 : \n"), outf)
+           Report("Test that estimates and standard errors are uncorrelated",
+                  outf)
+           if (is.na(y$cor.est))
+           {
+               Report("\ncannot be performed.\n\n", outf)
+           }
+           else
+           {
+               Report(c(": \nPearson correlation =", format(round(y$cor.est, 4),
+                                                            width=9),
+                        ", two-sided ",reportp(y$cor.pval,3), "\n\n"), sep="",
+                      outf)
+           }
+           Report(c("Test that all parameters are 0 : \n"), outf)
            Report(c("chi-squared =", format(round(y$Tsq, 4), width=9),
                     ", d.f. = ", y$n1, ", ",
                     reportp(y$pTsq, 3), "\n\n"), sep="", outf)
            Report(c("Estimated mean parameter",
-                    format(round(y[[2]]$coefficients[1, 1], 4), width=9),
-                    " (s.e.", format(round(y[[2]]$coefficients[1, 2], 4),
+                    format(round(y$regsummary$coefficients[1, 1], 4), width=9),
+                    " (s.e.", format(round(y$regsummary$coefficients[1, 2], 4),
                                      width=9), "), two-sided ",
-                    reportp(2 * pt(-abs(y[[2]]$coefficients[1, 3]),
-                               y$n1 - 1), 3), "\n\n"), sep="", outf)
+                    reportp(2 * pt(-abs(y$regsummary$coefficients[1, 3]),
+                                   y$n1 - 1), 3), "\n"), sep="", outf)
+           Report(c("based on IWLS modification of Snijders & Baerveldt (2003). ",
+                  "\n\n"), sep="", outf)
+           Report(c("Residual standard error",
+                    format(round(y$regsummary$stddev, 4), width=9)), outf)
+           Report("\nTest that variance of parameter is 0 :\n", outf)
+           Report(c("Chi-squared = ", format(round(y$Qstat, 4), width=9),
+                    " (d.f. = ", y$n1-1, "), ", reportp(y$pttilde, 3),
+                    "\n"), sep="", outf)
+           Report(c("based on IWLS modification of Snijders & Baerveldt (2003). ",
+                    "\n\n"), sep="", outf)
            Report("Fisher's combination of one-sided tests\n", outf)
            Report("----------------------------------------\n", outf)
            Report("Combination of right one-sided p-values:\n", outf)
@@ -232,8 +265,8 @@ print.sienaMeta <- function(x, file=FALSE, ...)
        else
        {
            Report(c("There were no data sets satisfying the bounds for",
-                   "this parameter.\n No combined output is given.\n"), outf)
-        }
+                    "this parameter.\n No combined output is given.\n"), outf)
+       }
    }, y=x)
     ##score tests
     if (any(x$scores))
@@ -252,8 +285,9 @@ print.sienaMeta <- function(x, file=FALSE, ...)
                         as.character(x$effects[1]), "\n"), sep="", outf)
                tmp <- paste("Data set ", 1:nrow(x), ", ", format(x$projname),
                             " : z = ", ifelse(is.na(x$scoretests), "NA",
-                        format(round(x$scoretests, 4), width=12)),
-                        "\n", sep="")
+                                              format(round(x$scoretests, 4),
+                                                     width=12)),
+                            "\n", sep="")
                Report(c(tmp, "\n"), sep="", outf)
                Report("Combination of right one-sided p-values:\n", outf)
                Report(c("Chi-squared = ", format(round(y$scoreplus, 4),
@@ -296,14 +330,14 @@ reportp <- function(p, ndec)
 plot.sienaMeta <- function(x, ...)
 {
     library(lattice)
-    tmp <- xyplot(se ~ theta|effects, data=x$thetadf, xlab="estimates",
-           ylab="standard errors", layout=c(4,4),
-           panel=function(x, y)
-       {
-           panel.xyplot(x, y)
-           panel.abline(0, 2)
-           panel.abline(0, -2)
-       }, scales="free")
+    tmp <- xyplot(theta ~ se|effects, data=x$thetadf, ylab="estimates",
+                  xlab="standard errors", layout=c(4,4),
+                  panel=function(x, y)
+              {
+                  panel.xyplot(x, y)
+                  panel.abline(0, qnorm(0.025))
+                  panel.abline(0, qnorm(0.975))
+              }, scales="free")
     tmp[!sapply(tmp$y.limits, function(x)all(is.na(x)))]
 }
 
@@ -397,7 +431,7 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
     }
     Report(c("================================= SIENA08 ",
              "================================================\n",
-             "Multilevel use of Siena algorithms according to",
+             "Multilevel use of Siena algorithms according to ",
              "Snijders & Baerveldt (2003) with extension\n",
              "=================================================",
              "=========================================\n\n"), sep="", outf)
@@ -423,9 +457,9 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
     {
         Report("-> No extra output requested\n", outf)
     }
-    ### RSiena version
+    ## RSiena version
     Report(c("\nThe RSiena Version of the first fit object is ",
-           x$thetadf$version[1], ".\n\n"), sep="", outf)
+             x$thetadf$version[1], ".\n\n"), sep="", outf)
     ## project names
     by(x$thetadf, x$thetadf$projname, function(x)
    {
@@ -433,20 +467,23 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
                 nrow(x), " parameters.\n"), sep="", outf)
        Report(c("The number of valid score tests found was ",
                 sum(is.na(x$scoretests)), ".\n"),
-                sep="",outf);
+              sep="", outf);
    })
     ##parameters:
     Report(c("\nA total of", nEffects, "parameters in", nProjects,
              "projects :\n"),   outf)
     Report(paste(format(1:length(effects)), ". " , effects, "\n", sep=""),
-             sep="", outf)
+           sep="", outf)
     Report(c("\nThe projects contain the parameters as follows",
-           "(1=present, 0=absent):\n\n"), outf)
+             "(1=present, 0=absent):\n\n"), outf)
     row1 <- c(1:nEffects)
     rows <- do.call(rbind, tapply(x$thetadf$effects,
-                                      x$thetadf$projname, function(x){
-                                          as.numeric(effects %in% x)
-                                      }))
+                                  x$thetadf$projname, function(x)
+                              {
+                                  as.numeric(effects %in% x)
+                              }
+                                  )
+                    )
     rows <- format(rbind(row1, rows), width=3)
     row2 <- rep(paste(rep("-", nchar(rows[1, 1])), collapse=""), nEffects)
     rows <- rbind(rows[1,], row2, rows[-1, ])
@@ -469,7 +506,7 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
                 format(round(max(abs(x$tconv)), 4), width=9), "\n"), outf)
        ## score tests
    })
-## results
+    ## results
 
     ## estimates
     Report(c("\n\n", paste(rep("=", 29), collapse=""),
@@ -501,25 +538,48 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
        {
            if (extra)
            {
-               Report(c("Snijders-Baerveldt (2003) method of combining",
-                        "estimates"), outf)
-               Report(c("\n---------------------------------------",
-                        "----------------\n"), sep="", outf)
+               Report(c("IWLS modification of Snijders-Baerveldt (2003) method ",
+                        "of combining estimates"), outf)
+               Report(c("\n--------------------------------------------",
+                        "---------------------------------\n"), sep="", outf)
                Report(c("This method assumes that true parameters and",
                         " standard errors are uncorrelated.\n",
-                        "This can be checked from the plots.\n"), sep="",
-                      outf)
+                        "This can be checked by the plot method ",
+                        "and the test below.\n\n"), sep="", outf)
            }
-           Report(c("\nTest that all parameters are 0 : \n"), outf)
+           Report("Test that estimates and standard errors are uncorrelated",
+                  outf)
+           if (is.na(y$cor.est))
+           {
+               Report("\ncannot be performed.\n\n", outf)
+           }
+           else
+           {
+               Report(c(": \nPearson correlation =", format(round(y$cor.est, 4),
+                                                            width=9),
+                        ", two-sided ",reportp(y$cor.pval,3), "\n\n"),
+                      sep="", outf)
+           }
+           Report(c("Test that all parameters are 0 : \n"), outf)
            Report(c("chi-squared =", format(round(y$Tsq, 4), width=9),
                     ", d.f. = ", y$n1, ", ",
                     reportp(y$pTsq, 3), "\n\n"), sep="", outf)
            Report(c("Estimated mean parameter",
-                    format(round(y[[2]]$coefficients[1, 1], 4), width=9),
-                    " (s.e.", format(round(y[[2]]$coefficients[1, 2], 4),
+                    format(round(y$regsummary$coefficients[1, 1], 4), width=9),
+                    " (s.e.", format(round(y$regsummary$coefficients[1, 2], 4),
                                      width=9), "), two-sided ",
-                    reportp(pt(y[[2]]$coefficients[1, 3],
-                               y$n1 - 1), 3), "\n\n"), sep="", outf)
+                    reportp(pt(y$regsummary$coefficients[1, 3],
+                               y$n1 - 1), 3), "\n"), sep="", outf)
+           Report(c("based on IWLS modification of Snijders & Baerveldt (2003). ",
+                  "\n\n"), sep="", outf)
+           Report(c("Residual standard error",
+                    format(round(y$regsummary$stddev, 4), width=9)), outf)
+           Report("\nTest that variance of parameter is 0 :\n",outf)
+           Report(c("Chi-squared = ", format(round(y$Qstat, 4), width=9),
+	            " (d.f. = ", y$n1-1, "), ", reportp(y$pttilde, 3),
+	            "\n"), sep="", outf)
+           Report(c("based on IWLS modification of Snijders & Baerveldt (2003).",
+                  "\n\n"), sep="", outf)
            Report("Fisher's combination of one-sided tests\n", outf)
            Report("----------------------------------------\n", outf)
            Report("Combination of right one-sided p-values:\n", outf)
@@ -534,8 +594,8 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
        else
        {
            Report(c("There were no data sets satisfying the bounds for",
-                   "this parameter.\n No combined output is given.\n"), outf)
-        }
+                    "this parameter.\n No combined output is given.\n"), outf)
+       }
    }, y=x)
     ##score tests
     if (any(x$scores))
@@ -553,9 +613,10 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
                Report(c("\n", "(", i, ")   ",
                         as.character(x$effects[1]), "\n"), sep="", outf)
                tmp <- paste("Data set ", 1:nrow(x), ", ", format(x$projname),
-                            " : z = ", ifelse(is.na(x$scoretests), "NA",
-                        format(round(x$scoretests, 4), width=12)),
-                        "\n", sep="")
+                            " : z = ",
+                            ifelse(is.na(x$scoretests), "NA",
+                                   format(round(x$scoretests, 4), width=12)),
+                            "\n", sep="")
                Report(c(tmp, "\n"), sep="", outf)
                Report("Combination of right one-sided p-values:\n", outf)
                Report(c("Chi-squared = ", format(round(y$scoreplus, 4),
@@ -570,6 +631,5 @@ print.summary.sienaMeta <- function(x, file=FALSE, extra=TRUE, ...)
            }
        }, y=x)
     }
-
 }
 
