@@ -48,6 +48,7 @@ public:
 	double similarityMean() const;
 	void similarityMean(double similarityMean);
 	void similarityMeans(double similarityMean, std::string networkName);
+	virtual double observedDistribution(int value, int observation) const;
 	void calculateProperties();
 
 private:
@@ -77,6 +78,12 @@ private:
 
 	// The alter similarity means for each network (to be passed from outside)
 	std::map<std::string, double> lsimilarityMeans;
+
+	// The distribution of observed values at each observation.
+	// lobservedDistributions[observation][value] stores the frequency of
+	// the given value at the given observation.
+
+	map<int, double> * lobservedDistributions;
 };
 
 }
