@@ -21,16 +21,16 @@ namespace siena
  * Outdegree effect defined as the product of the ego with ths number of
  * its outward neighbors (with respect to a certain network).
  */
-	class OutdegreeEffect : public NetworkDependentBehaviorEffect
+class OutdegreeEffect : public NetworkDependentBehaviorEffect
 {
 public:
 	OutdegreeEffect(const EffectInfo * pEffectInfo);
 
 	virtual double calculateChangeContribution(int actor,
-		int difference) const;
-	virtual double evaluationStatistic(double * currentValues) const;
+		int difference);
 	virtual double endowmentStatistic(const int * difference,
-		double * currentValues) const;
+		double * currentValues);
+	virtual double egoStatistic(int ego, double * currentValues);
 };
 
 }
